import java.util.*;
import greenfoot.*;

/**
 * Elternklasse von Peetie. Wird für diverse Variablen + Methoden benutzt, die Peetie erbt, damit die Klasse Peetie schlank bleibt und von den Schüler/innen einfach bedient werden kann.
 */
public abstract class AI_Robot extends Actor
{
    // Werte nach Belieben ändern:
    protected int peetieStartposX = 425; // Achtung: Weltgröße und Gridgröße berücksichtigen (World-Klasse Mars)!
    protected int peetieStartposY = 225;
    protected int peetieDirection = 0; // Schaut nach Osten
    protected int delay = 40;   // Wie lange wartet Peetie zwischen einzelnen Aktionen (gehe Links, geheRechts...)
    // Möglich auch warten(int delay)
    protected String peetieDefaultImage = "peetie_runter_50.png";

    // Andere Variablen, nicht ändern
    protected Mars mars; // Wird in der addedToWorld-Methode initialisiert
    protected int gridSize; // Wird in der addedToWorld-Methode initialisiert
    protected int dx, dy; // Für Sensoren (istFelsVoraus etc.) - In welche Richtung/wie weit entfernt liegt das nächste Kollisions Objekt?
    protected int quarzX = 225, quarzY = 125; // Quarzmunition herstellen
    protected int anzahlFighterdrohnen = 7; // Ep05M03

    protected boolean buttonsSindEingeblendet = true;
    protected boolean missionBeendet = false;

    public AI_Robot() {
        this.setRotation(this.peetieDirection);
    }

    public void act() {
        // Wenn die act-Methode als Loop laufen soll, folgende Zeile entkommentieren:
        // this.run();
    }

    public void runOrganize() {
        // Blendet Buttons vor Beginn von run() aus und danach wieder ein
        this.mars.buttonsVerstecken();
        this.warten(1);
        this.run();
        this.warten(80);
        this.mars.buttonsZeigen();
    }
    /** 
     * #########################################################
     * #########################################################
     * ################# PEETIE-BEFEHLE ########################
     * #########################################################
     * #########################################################
     * werden von Schueler/innen verwendet (geheRechts usw.)
     */

    /**
     * #################### BEWEGEN ######################
     * Richtung, Bild, Bildrotation, Sound
     */
    public void geheRechts() { this.bewegen(0, "peetie_rechts_50.png", 0, "STEP1"); }

    public void geheLinks() { this.bewegen(180, "peetie_links_50.png", 180, "STEP2"); }

    public void geheHoch() { this.bewegen(270, "peetie_hoch_50.png", 90, "STEP3"); }

    public void geheRunter() { this.bewegen(90, "peetie_runter_50.png", 270, "STEP4"); }

    /**
     * #################### LOCH GRABEN ######################
     */
    public void lochGraben() {
        Item i = new Item();
        i.setImage("peetie_runter_50_loch.png");
        Sound.play("GRABEN");
        this.getWorld().addObject(i, this.getX(), this.getY());
    }

    /**
     * #################### FELSEN + KRATER UMRUNDEN ######################
     */
    public boolean istFelsVoraus() {
        return this.istObjektVoraus(Fels.class);
    }

    public boolean istKraterVoraus() {
        return this.istObjektVoraus(Krater.class);
    }

    public void aussenrumgehen() {
        // funktioniert nur, keine 2 Felsen nebeneinander liegen
        int dx = 0, dy = 0;
        switch(this.getRotation()) {
            case 0: // rechts
                this.dy = -this.gridSize;
                this.dx = this.gridSize;
                break;
            case 180: // links
                this.dy = -this.gridSize;
                this.dx = -this.gridSize;
                break;
            case 90: // unten
                this.dy = this.gridSize;
                this.dx = this.gridSize;
                break;
            case 270: // hoch
                this.dy = -this.gridSize;
                this.dx = this.gridSize;
                break;
        }
        this.geheRunter();
        this.geheLinks();
        this.geheLinks();
        this.geheHoch();
    }

    /**
     * #################### RAKETEN SAMMELN ######################
     */
    public boolean raketeWirdBeruehrt() {
        return this.isTouching(Rakete.class);
    }

    public void raketeAufnehmen() {
        if(this.raketeWirdBeruehrt()) {
            Rakete r = (Rakete)this.getOneIntersectingObject(Rakete.class);
            if(!r.getRaketeIstSicher()) {
                this.missionEnde("Rakete war ein Sprengkörper von Syberdrone - #FAIL", "EXPLOSION");
            } else {
                this.removeTouching(Rakete.class);
                Sound.play("RAKETENEHMEN");
                if(this.alleRaketenGesammelt()) {
                    /*
                     * Hier zusätzliche Bedingungen möglich, bspw. wenn Peetie das Trümmerfeld
                     * überwunden haben muss:
                     * if(this.getY() > 500) …
                     */
                    if(true) {
                        this.missionEnde("Alle Raketen gesammelt, perfekt!", "YEAH!");
                    }
                }
            }
        } else {
            this.missionEnde("Keine Rakete. Peetie stürzt in einer Endlosschleife ab.", "ENDLOSSCHLEIFE");
        }
    }

    public boolean raketeIstSicher() {
        Rakete r = (Rakete)this.getOneIntersectingObject(Rakete.class);
        if(r != null) {
            if(r.getRaketeIstSicher()) {
                return true;
            } else {
                return false;
            }
        } else {
            this.missionEnde("Keine Rakete. Peetie kontrolliert wieder und wieder und stürzt ab.", "ENDLOSSCHLEIFE"); 
        }
        return false;
    }

    public void entschaerfen() {
        Rakete r = (Rakete)this.getOneIntersectingObject(Rakete.class);
        if(r != null) {
            if(r.getRaketeIstSicher()) { // es wird fälschlicherweise entschärft
                this.missionEnde("Beim Versuch, eine echte Rakete zu entschärfen, fliegt Peetie in die Luft. #FAIL", "EXPLOSION"); 
            } else {
                r.setRaketeIstSicher(true);
                Sound.play("ENTSCHAERFEN");
                this.getWorld().showText("Rakete erfolgreich entschärft!!", this.getWorld().getWidth()/2, this.getWorld().getHeight()/2);
                this.warten(50);
                this.getWorld().showText("", this.getWorld().getWidth()/2, this.getWorld().getHeight()/2);
            }
        } else {
            this.missionEnde("Keine Rakete. Peetie entschärft sich aus Versehen selbst. #FAIL", "EXPLOSION");
        }
    }

    /**
     * #################### FEINDLICHE BASIS ANGREIFEN ######################
     */
    public boolean basisErreicht() {
        return this.isTouching(Basis.class);
    }

    public int holeAnzahlFighterdrohnen() {
        // Diese Methode fakt das komplette Spielgeschehen; wenn die Drohnen
        // wirklich auf der World sind und gezählt, beschossen … werden,
        // hängt sich alles auf (while-Schleife in Peeties run()). Deshalb dieser Fake hier:
        this.warten(4);

        if(Greenfoot.getRandomNumber(1000) > 900) { // Peetie schießt - nur geräusch + optik
            Sound.play("ABSCHUSS");
            Quarzmunition r = new Quarzmunition();
            this.getWorld().addObject(r, this.getX(), this.getY());
            r.getImage().scale(r.getImage().getWidth()/2, r.getImage().getHeight()/2);
            boolean drohneGetroffen = r.fliegen();
            this.getWorld().removeObject(r);
            if(drohneGetroffen) { //zufällige Drohnentreffer
                this.anzahlFighterdrohnen--;
                this.baldFertigSpruchWurdeAusgegeben = false;
                for(Fighterdrohne f : this.getWorld().getObjects(Fighterdrohne.class)) {
                    this.getWorld().removeObject(f);
                }
                for(int i = 1; i <= this.anzahlFighterdrohnen; i++) {
                    this.getWorld().addObject(new Fighterdrohne(), 1, 1);
                }
            }
        }
        return this.anzahlFighterdrohnen;
    }
    protected boolean baldFertigSpruchWurdeAusgegeben = false;
    public void baldFertig() {
        if(!this.baldFertigSpruchWurdeAusgegeben) {
            this.textEinfliegen("BALD FERTIG!");
            this.baldFertigSpruchWurdeAusgegeben = true;
        }
    }

    /**
     * #################### TEXT EINLESEN / AUSGEBEN ######################
     */
    public String textEinlesen() {
        return Greenfoot.ask("");
    }

    public double zahlEinlesen() {
        return Double.parseDouble(this.textEinlesen());
    }

    private String inputToString(String text) {
        return text;
    }

    private String inputToString(int text) {
        return Integer.toString(text);
    }

    private String inputToString(double text) {
        return Double.toString(text);
    }

    // User: textAusgeben() - packt ihn mit showtext mitten auf die Welt
    public void textAusgeben(String text) {
        this.textShowen(this.inputToString(text));    
    }

    public void textAusgeben(int text) {
        this.textShowen(this.inputToString(text));    
    }

    public void textAusgeben(double text) {
        this.textShowen(this.inputToString(text));    
    }
    // textausgeben mit koordinaten
    public void textAusgeben(String t, int x, int y) {
        this.getWorld().showText(t, x, y);
    }

    public void textAusgeben(int t, int x, int y) {
        this.getWorld().showText(Integer.toString(t), x, y);
    }
    private void textShowen(String t) {
        this.getWorld().showText(t, this.mars.getWidth()/2, this.mars.getHeight()/2 - 50);
    }
    // User: ausgabe() - lässt ihn einfliegen, kurz stehen und dann wieder raus

    public void ausgabe(String text) {
        this.textEinfliegen(this.inputToString(text));
    }

    public void ausgabe(int text) {
        this.textEinfliegen(this.inputToString(text));
    }

    public void ausgabe(double text) {
        this.textEinfliegen(this.inputToString(text));
    }

    private void textEinfliegen(String t) {
        GreenfootImage g = new GreenfootImage(t, 20, Color.WHITE, Color.RED);
        Label l = new Label();
        l.setImage(g);
        this.getWorld().addObject(l, this.getWorld().getWidth(),200);
        for(int x = this.getWorld().getWidth(); x >= this.getWorld().getWidth()/2; x = x - 15) {
            l.setLocation(x, l.getY());
            this.warten(1);
        }
        Sound.play("BING");
        this.warten(200);
        for(int x = this.getWorld().getWidth()/2; x >= 0; x = x - 15) {
            l.setLocation(x, l.getY());
            this.warten(1);
        }
        this.getWorld().removeObject(l);
    }

    /**
     * #################### TASTATUR ######################
     */
    public boolean tasteGedrueckt(String t) {
        if(Greenfoot.isKeyDown(t)) {
            return true;
        } else {
            return false;
        }
    }

    public void leertasteGedrueckt() {
        if(Greenfoot.isKeyDown("space")) {
            System.out.println("Bammm abgefeuert");
        }
    }

    public void raketeAbfeuern() {
        Quarzmunition r = new Quarzmunition();
        this.getWorld().addObject(r, this.getX(), this.getY());
        r.getImage().scale(r.getImage().getWidth()/2, r.getImage().getHeight()/2);
        Sound.play("ABSCHUSS");
        r.fliegen();
        this.getWorld().removeObject(r);
    }

    /**
     * #################### MUNITION HERSTELLEN / RECHNEN ######################
     */
    public void stelleQuarzmunitionHer() {
        Quarzmunition q = new Quarzmunition();
        this.getWorld().addObject(q, this.quarzX, this.quarzY);
        Sound.play("BING");

        if(this.quarzX > this.getWorld().getWidth() - 150) {
            this.quarzX = 150;
            this.quarzY += 50;
        }
        this.quarzX += 65;
    }

    /* 
     * #########################################################
     * #########################################################
     * ################# PRIVATE METHODEN ######################
     * #########################################################
     * #########################################################
     * werden von Standard-Methoden benutzt,
     * werden aber von Schüler/innen nicht benutzt 
     * 
     */
    private void bewegen(int neueAusrichtung, String neuesBild, int bildRotation, String sound) {
        if(!this.missionBeendet) {
            this.setRotation(neueAusrichtung);
            this.setImage(neuesBild);
            this.getImage().rotate(bildRotation);
            this.move(this.gridSize);
            Sound.play(sound);
            this.kollisionenChecken();
            this.warten(); 
        }
    }

    private void setDxDy() { // Ermittelt Offset zum nächsten Objekt in Blickrichtung
        this.dx = 0;
        this.dy = 0;
        switch(this.getRotation()) {
            case 0: // rechts
                this.dx = this.gridSize;
                break;
            case 180: // links
                this.dx = -this.gridSize;
                break;
            case 90: // unten
                this.dy = this.gridSize;
                break;
            case 270: // oben
                this.dy = -this.gridSize;
                break;
            default:
                break;

        }
    }

    private boolean istObjektVoraus(Class c) {
        this.setDxDy();
        if(this.getOneObjectAtOffset(this.dx, this.dy, c) != null) {
            return true;
        } else {
            return false;
        }
    }

    private void kollisionenChecken() {
        // Verwaltet automatisches Einsammeln von Batterien etc. und destruktive Kollisionen mit Felsen usw.
        // Je nach Szenario können einzelne Kollisionsprüfungen abgeschaltet werden (auskommentieren):
        this.checkBatterieKollision();
        this.checkTruemmerteilKollision();
        this.checkPeetieBruchstueckKollision();
        this.checkFelsKollision();
        this.checkKraterKollision();
    }

    private void checkPeetieBruchstueckKollision() {
        if(this.isTouching(PeetieBruchstueck.class)) {
            PeetieBruchstueck pb = (PeetieBruchstueck)this.getOneIntersectingObject(PeetieBruchstueck.class);
            this.missionEnde(pb.getBezeichnung() + " gefunden!", "YEAH!");
        }
    }

    private void checkBatterieKollision() {
        if(this.isTouching(Batterie.class)) {
            Sound.play("BATTERYEATEN");
            this.removeTouching(Batterie.class);
            if(alleBatterienGesammelt()) 
            { // alle Batterien eingesammelt
                /*
                 * Hier zusätzliche Bedingungen möglich, bspw. wenn Peetie das Trümmerfeld
                 * überwunden haben muss:
                 * if(this.getY() > 500) …
                 * (statt if(true) …)
                 */
                if(true) {
                    this.missionEnde("Alle Batterien gesammelt, perfekt!", "YEAH!");
                }
            }
        }
    }

    private boolean alleBatterienGesammelt() {
        List<Batterie> batterienAufWelt = this.getWorld().getObjects(Batterie.class);
        return batterienAufWelt.size() <= 0 ? true : false;
    }

    private boolean alleRaketenGesammelt() {
        List<Rakete> raketenAufWelt = this.getWorld().getObjects(Rakete.class);
        return raketenAufWelt.size() <= 0 ? true : false;
    }

    private void checkTruemmerteilKollision() {
        if(this.isTouching(Truemmerteil.class)) {
            this.removeTouching(Truemmerteil.class);
            this.missionEnde("KOLLIDIERT MIT TRUEMMERTEIL!! #FAIL", "EXPLOSION");
        }
    }

    private void checkKraterKollision() {
        if(this.isTouching(Krater.class)) {
            this.removeTouching(Krater.class);
            this.missionEnde("IN EINEN KRATER GESTÜRZT … #FAIL", "ABSTURZ");
        }
    }

    private void checkFelsKollision() {
        if(this.isTouching(Fels.class)) {
            this.removeTouching(Fels.class);
            this.missionEnde("VOLL AUF DEN FELSEN GEPRALLT! #FAIL", "STEINSCHLAG");
        }
    }

    private void missionEnde(String spruch, String sound) {
        this.missionBeendet = true;
        this.warten(8);
        Sound.play(sound);
        this.getWorld().showText(spruch, this.mars.getWidth()/2, this.mars.getHeight()/2);
        this.warten(50);
        Sound.stopAll();        
    }

    private void warten() {
        Greenfoot.delay(this.delay);
    }

    private void warten(int del) {
        Greenfoot.delay(del);
    }

    /**
    * SETUP-/INITIALIZE-functions
    */
    public void addedToWorld(World w) {
        this.startpositionEinnehmen(w);
        this.mars = (Mars)w;
        this.gridSize = this.mars.getGridSize();
    }
    public void startpositionEinnehmen(World w) {
        this.setLocation(this.peetieStartposX, this.peetieStartposY);
        this.setImage(this.peetieDefaultImage);
    }

    /* Andere Methoden, die Schüler/innen nicht brauchen: */
    public void run() {
        // Wird in Peetie überschrieben
    }
    /* Getter, Setter */
    public int getPeetieStartposX() {
        return this.peetieStartposX;
    }

    public int getPeetieStartposY() {
        return this.peetieStartposY;
    }

    public int getDelay() {
        return this.delay;
    }

    public void setDelay(int delay) {
        this.delay = delay;
    }
}
